import { AstroError, AstroErrorData } from "../errors/index.js";
import { getParamsAndProps } from "./params-and-props.js";
const clientLocalsSymbol = Symbol.for("astro.locals");
async function createRenderContext(options) {
  const request = options.request;
  const pathname = options.pathname ?? new URL(request.url).pathname;
  const [params, props] = await getParamsAndProps({
    mod: options.mod,
    route: options.route,
    routeCache: options.env.routeCache,
    pathname,
    logger: options.env.logger,
    ssr: options.env.ssr
  });
  const context = {
    ...options,
    pathname,
    params,
    props,
    preferredLocale: options.preferredLocale,
    preferredLocaleList: options.preferredLocaleList
  };
  Object.defineProperty(context, "locals", {
    enumerable: true,
    get() {
      return Reflect.get(request, clientLocalsSymbol);
    },
    set(val) {
      if (typeof val !== "object") {
        throw new AstroError(AstroErrorData.LocalsNotAnObject);
      } else {
        Reflect.set(request, clientLocalsSymbol, val);
      }
    }
  });
  return context;
}
function parseLocale(header) {
  if (header === "*") {
    return [{ locale: header, qualityValue: void 0 }];
  }
  const result = [];
  const localeValues = header.split(",").map((str) => str.trim());
  for (const localeValue of localeValues) {
    const split = localeValue.split(";").map((str) => str.trim());
    const localeName = split[0];
    const qualityValue = split[1];
    if (!split) {
      continue;
    }
    if (qualityValue && qualityValue.startsWith("q=")) {
      const qualityValueAsFloat = Number.parseFloat(qualityValue.slice("q=".length));
      if (Number.isNaN(qualityValueAsFloat) || qualityValueAsFloat > 1) {
        result.push({
          locale: localeName,
          qualityValue: void 0
        });
      } else {
        result.push({
          locale: localeName,
          qualityValue: qualityValueAsFloat
        });
      }
    } else {
      result.push({
        locale: localeName,
        qualityValue: void 0
      });
    }
  }
  return result;
}
function computePreferredLocales(request, locales) {
  const acceptHeader = request.headers.get("Accept-Language");
  const result = [
    void 0,
    void 0
  ];
  if (acceptHeader) {
    const parsedResult = parseLocale(acceptHeader).filter((browserLocale) => {
      if (browserLocale.locale !== "*") {
        return locales.includes(browserLocale.locale);
      }
      return true;
    }).sort((a, b) => {
      if (a.qualityValue && b.qualityValue) {
        if (a.qualityValue > b.qualityValue) {
          return -1;
        } else if (a.qualityValue < b.qualityValue) {
          return 1;
        }
      }
      return 0;
    });
    if (parsedResult.length === 1 && parsedResult.at(0).locale === "*") {
      result[0] = locales;
    } else {
      result[0] = parsedResult.map((item) => {
        return item.locale;
      });
    }
    const firstResult = parsedResult.at(0);
    if (firstResult) {
      if (firstResult.locale !== "*") {
        result[1] = firstResult.locale;
      }
    }
  }
  return result;
}
export {
  computePreferredLocales,
  createRenderContext,
  parseLocale
};
